#!/usr/bin/perl

#
# add.cgi
#
# Upload an image to the database.
#
# Steps are:
#
# - If no upload is present, give the upload form
# - Verify the mime type
# - Copy to a temporary file
# - Insert the file into the database
# - Direct the user to the edit form
#
# if user has uploaded an archive, then extract archive at
# step 2.5 and loop to step 4 for each image.

use HTTP::Request;
use LWP::UserAgent;
use File::MMagic;
use Archive::Tar;
use Archive::Zip;
use Digest::MD5;  # to get md5sum which will prevent duplicates


db_connect();

$tmpdir = "$RealDir/scripts/tmp/";

$Owner = Userverify($q);

$albref = $q->param('albref');
$Submit = $q->param('submit');
$Keywords = $q->param('keywords');
$Ref = $q->param('ref');
$Referer = $ENV{'HTTP_REFERER'};

if ($ENV{REQUEST_METHOD} eq "POST") {
	if ($Submit ne "Upload") {
		print "Location: $Ref\n\n";
		exit(0);
	}
	ProcessUpload();
} else {
	PrintHeader($Owner, "PhotoShelf - Upload", "onunload=\"OnUnload();\"", "Upload image(s)");
	print  "<center>", QuickAdd(undef,0,$Referer) , "</center>";
	PrintFooter();
	exit;
}

sub ProcessUpload {
my($filename,$mime,$newfile,$id,$file,$bytesread,$buffer);

	($file) = $q->param('imagefile');
	($url) = $q->param('imageurl');
	$forminfo = ParseForm();
	if ($url =~ /^$/ && $file =~ /^$/) {
		print "\n\nError: please enter either a URL or a file";
		exit(0);
	}
	if ($url !~ /^$/) {
		ProcessUrl($url);
	}

	# take off the directory information
	$filename = strip_dir($file);

	# Make sure we have a file

	if ($filename =~ /^$/) {
		print QuickAdd(undef,0);
		exit;
	}

	if (db_connect() == -1) {
		die;
	}
	
	# Copy into a temporary file

	$newfile = "$tmpdir" . "/" . $$ . ".img";
	open (OUTFILE,">$newfile") or die "Can't open $newfile: $!";
	while ($bytesread=read($file,$buffer,1024)) {
		print OUTFILE $buffer;
	}
	close OUTFILE;

	# Insert pic;

	my $mm = new File::MMagic;
	$type = $mm->checktype_filename($newfile);
	if ($type =~ /zip/ || $type =~ /tar/) {
		$id = ExtractArchive($newfile);
	} else {
		$id = InsertImage($filename, $newfile);
		unlink $newfile;
		if ($id !~ /^[0-9]+$/) {
			InsertError($id);
			exit(0);
		}
	}

	print "Location: edit.cgi?id=$id&refalbum=$defaultalbum\n\n";
	exit(0);
}

sub ProcessUrl {
my($url) = @_;
my($first) = "";
my(%lfiles) = ();

	if ($url !~ /^http:\/\//) {
		print "\n\nOnly http urls suppoerted!\n";
		exit(0);
	}
	foreach $ur (split(/\s+/, $url)) {
		$hdrs = new HTTP::Headers(Accept => '*/*');
		$ul = new URI::URL($ur);
		$req = new HTTP::Request(GET, $ul, $hdrs);
		$ua = new LWP::UserAgent;
		$resp = $ua->request($req);
		if ($resp->is_success) {
			my($num) = int(rand(10000));
			$newfile = "$tmpdir" . "/" . $$ . $num . ".img";
			open (OUTFILE,">$newfile") or die "Can't open $newfile: $!";
			print OUTFILE $resp->content;
			close OUTFILE;
			# Insert pic;
			if ($ur =~ /\/([^\/]+)$/) {
				$filename = $1;
			}
			$lfiles{$newfile} = $filename;

		} else {
			warn "Error: URL $ur could not be retrieved, error was:" . $resp->message;
		}
	}
	$count = 0;
	foreach $newfile (keys %lfiles) {
			$filename = $lfiles{$newfile};
			my $mm = new File::MMagic;
			$res = $mm->checktype_filename("$newfile");
			if ($res =~ /zip/ || $res =~ /tar/) {
				$id = ExtractArchive($newfile);
			} elsif ($res eq "image/jpeg") {
				$id = InsertImage($filename, $newfile);
			} else {
				BadType();
			}
			$first = $id, unless($first > 0);
			$count ++;
			unlink($newfile);
	}
	if ($count == 1) {
		print "Location: edit.cgi?id=$id&refalbum=$albref\n\n";
	} else {
#		$defaultalbum = AlbumNameId($Owner, "Default");
		print "Location: albumshow?id=$albref\n\n";
	}
	exit(0);
}
sub BadType {

	print "<b>Error: $filename is not a jpeg image or tar/zip file!</b><br>";
	exit;
}

sub InsertImage {
my($name,$file) = @_;
my($info,$qry,$res,$id);

	my $md5 = md5_check($file);
	if ($md5 eq "") {
		return "File already exists (same md5 sum)";
	}
	$info = get_image_info($file);
	$info->{'md5'} = $md5;
	$info->{'filename'} = $name;
	$info->{'keywords'} = $Keywords;
	$info->{'owner'} = $Owner;
	$qry = sprintf("SELECT id FROM format WHERE descr = '%s'",
		$info->{'format'});
	($info->{'format'}) = $Db_Conn->selectrow_array($qry);
	$id = database_insert($info);
	if (!$id || $id == -1) {
		return "Error inserting $file into database!\n";
	}
	db_image_owner($id, $Owner);
	$res = repository_insert($file, $id);
	if (!$res) {
		database_delete($stripped);
		return "Error inserting $file into repository!";
	}
	$qry = sprintf("UPDATE photos SET directory = '%s', albumrefcount = 0 WHERE imageid = %d",
				$res, $id);
	$res = $Db_Conn->do($qry);
	$defaultalbum = AlbumNameId($Owner, "Default");
	AlbumAdd($defaultalbum, $id);
	return $id;
}

sub md5_check {
my($file) = @_;

	open (PIC, "$file") or warn "Cannot open $file: $!";
	my $img = "";
	while (<PIC>) {
		$img .= $_;
	}
	close PIC;
	my $sum = Digest::MD5::md5_hex($img);
	my @res = $Db_Conn->selectrow_array("SELECT imageid,md5sum FROM photos WHERE owner = '$Owner' AND md5sum = '$sum'");
	if ($res[0] !~ /^$/) {
		warn "md5sum matches image $res[0]";
		return "";
	} else {
		return $sum;
	}
}

sub ParseForm {
my ($info, $bit);

	foreach $bit ("descr", "country", "city", "copyright", "caption") {
		$info{$bit} = $q->param($bit), if (defined($q->param($bit)));
	}
}

# extract files from an archive

sub ExtractArchive {
my($file) = @_;
my $ret, $mm, $cwd, $res;

	use File::Find;

	$mm = new File::MMagic;
	$res = $mm->checktype_filename("$file");
	mkdir ("$tmpdir/unarchived.$$", 0711) or die "Can't create directory: $!";
	chomp($cwd = `pwd`);
	chdir "$tmpdir/unarchived.$$" or warn "Can't chdir!";
	if ($res =~ /gzip/ || $res =~ /tar/) {
		ExtractTar($file);
	} elsif ($res =~ /x-zip/) {
		ExtractZip($file);
	}
RECURSE:
	find sub {
		my $fname = $_;
		my($file) = $File::Find::name;
		if (-f $file) {
			$res = $mm->checktype_filename("$file");
			if ($res eq "image/jpeg") {
				push (@Files, "$file");
				$ret = InsertImage($fname, $file);
			}
			unlink $file;
		}
	}, "$tmpdir/unarchived.$$/";
	chdir $cwd;
	rmdir  "$tmpdir/unarchived.$$";
	return $ret;
}

sub ExtractTar {
my($file) = @_;

	my $res = Archive::Tar->extract_archive($file);
	if (!defined($res)) {
		warn "Error- cannot untar $file: " . Archive::Tar::error;
		return 0;
	}
	return 1;
}

sub ExtractZip {
my($file) = @_;

	my $zip = Archive::Zip->new();
	die "Error reading $file", if $zip->read($file) != AZ_OK;

	my @members = $zip->memberNames();
	foreach my $name (@members) {
		$res = $zip->extractMember($name);
	}
}

sub InsertError {
my($Error) = @_;

	PrintHeader($Owner,"Error adding file", "", "Error adding file");
	print <<EOF;
<center><br>
There was an error adding the image to the database: <p><b>$Error</b><p>Please rectify this.
</center><br>
EOF
PrintFooter();
}
